#!/bin/ksh

[ -z "$GATEWAY" ] && GATEWAY=srvx9.img.univie.ac.at
[ -z "$DESTINATION" ] && DESTINATION=leo@genericSftp
[ -z "$ECUID" ] && ECUID=lh0
[ -z "$ECPATH" ] && ECPATH=spatlh00/

#[ -z "$GATEWAY" ] && GATEWAY=ctbto4.ctbto.org
#[ -z "$DESTINATION" ] && DESTINATION=atmops@ops
#[ -z "$ECUID" ] && ECUID=cbb
#[ -z "$ECPATH" ] && ECPATH=ctbto/

echo 'ECMWFDATA v4.0 is installed for:'
echo Gateway     $GATEWAY 
echo Destination $DESTINATION 
echo ECMWF user ID $ECUID 
echo ECMWF scratch path /scratch/ms/$ECPATH/$ECUID/
echo ECMWF home path /home/ms/$ECPATH/$ECUID/

echo 'Note: These settings can be changed via environment variables'
echo '$GATEWAY $DESTINATION  $ECUID  $ECPATH'

cat >header<<EOF

# ON ECGB:
# start with ecaccess-job-submit -queueName ecgb NAME_OF_THIS_FILE  on gateway server
# start with sbatch NAME_OF_THIS_FILE directly on machine

#SBATCH --workdir=/scratch/ms/$ECPATH/$ECUID/
#SBATCH --qos=normal
#SBATCH --job-name=flex_ecmwf
#SBATCH --output=flex_ecmwf.%j.out
#SBATCH --error=flex_ecmwf.%j.out
#SBATCH --mail-type=FAIL
#SBATCH --time=12:00:00

EOF

SERVER=`uname -n`
if [[ ${SERVER} != ecgate && ${SERVER} != ecgb ]] ; then

echo "Software copied from server ${SERVER}" to ecgb

export ECDOMAIN=ECHOME
ksh upload_source V4

# modify headers of operational scripts
sed "s,spatlh00/,${ECPATH},"  ecmwf_idc_ops_ecgate-anfc3hr-gauss.tmp | sed "s,lh0,${ECUID}," > ecmwf_idc_ops_ecgate-anfc3hr-gauss
sed -i '/queue/ r header' ecmwf_idc_ops_ecgate-anfc3hr-gauss
sed "s,spatlh00/,${ECPATH}," ecmwf_idc_ops_ecgate-anfc3hr-gauss-399.tmp | sed "s,lh0,${ECUID}," >ecmwf_idc_ops_ecgate-anfc3hr-gauss-399
sed -i '/queue/ r header' ecmwf_idc_ops_ecgate-anfc3hr-gauss-399
sed "s,spatlh00/,${ECPATH}," ecmwf_idc_ops_ecgate-anfcan3hr-799.tmp | sed "s,lh0,${ECUID}," > ecmwf_idc_ops_ecgate-anfcan3hr-799
sed -i '/queue/ r header' ecmwf_idc_ops_ecgate-anfcan3hr-799
sed  "s,lh0,${ECUID},"  CONTROL_OPS_V2.tmp | sed "s,srvx9.img.univie.ac.at,${GATEWAY}," | sed "s,leo@genericSftp,${DESTINATION}," >CONTROL_OPS_V2
sed  "s,lh0,${ECUID},"  CONTROL_OPS_V4.tmp | sed "s,srvx9.img.univie.ac.at,${GATEWAY}," | sed "s,leo@genericSftp,${DESTINATION}," >CONTROL_OPS_V4

# automatically set gateway and destination for on demand scripts
sed "s,srvx9.img.univie.ac.at,${GATEWAY}," flex_ecmwf_91_finegrid_hpce.tmp | sed "s,leo@genericSftp,${DESTINATION}," > flex_ecmwf_91_finegrid_hpce
sed "s,srvx9.img.univie.ac.at,${GATEWAY}," flex_ecmwf_91_finegrid_ecgate.tmp | sed "s,leo@genericSftp,${DESTINATION}," > flex_ecmwf_91_finegrid_ecgate
sed -i '/queue/ r header' flex_ecmwf_91_finegrid_ecgate
sed "s,srvx9.img.univie.ac.at,${GATEWAY}," flex_ecmwf_91_eta_ecgate.tmp | sed "s,leo@genericSftp,${DESTINATION}," > flex_ecmwf_91_eta_ecgate
sed -i '/queue/ r header' flex_ecmwf_91_eta_ecgate
sed "s,srvx9.img.univie.ac.at,${GATEWAY}," flex_ecmwf_91_global_1.0_ecgate.tmp | sed "s,leo@genericSftp,${DESTINATION}," > flex_ecmwf_91_global_1.0_ecgate
sed -i '/queue/ r header' flex_ecmwf_91_global_1.0_ecgate

ecaccess-file-mkdir flex_extract_ecgate_V4
ecaccess-file-put flex_ecmwf_91_finegrid_hpce flex_extract_ecgate_V4/flex_ecmwf_91_finegrid_hpce
ecaccess-file-put flex_ecmwf_91_finegrid_ecgate flex_extract_ecgate_V4/flex_ecmwf_91_finegrid_ecgate
ecaccess-file-put flex_ecmwf_91_eta_ecgate flex_extract_ecgate_V4/flex_ecmwf_91_eta_ecgate
ecaccess-file-put flex_ecmwf_91_global_1.0_ecgate flex_extract_ecgate_V4/flex_ecmwf_91_global_1.0_ecgate

ecaccess-file-put  ecmwf_idc_ops_ecgate-anfc3hr-gauss flex_extract_ecgate_V4/ecmwf_idc_ops_ecgate-anfc3hr-gauss
ecaccess-file-put ecmwf_idc_ops_ecgate-anfc3hr-gauss-399 flex_extract_ecgate_V4/ecmwf_idc_ops_ecgate-anfc3hr-gauss-399
ecaccess-file-put  ecmwf_idc_ops_ecgate-anfcan3hr-799 flex_extract_ecgate_V4/ecmwf_idc_ops_ecgate-anfcan3hr-799
ecaccess-file-put CONTROL_OPS_V4 flex_extract_ecgate_V4/CONTROL_OPS_V4
ecaccess-file-put CONTROL_OPS_V2 flex_extract_ecgate_V4/CONTROL_OPS_V2

export ECDOMAIN=ECSCRATCH
ecaccess-file-mkdir ms_sms_output_V4
export ECDOMAIN=ECHOME

echo 'submitting example  on demand script flex_ecmwf_91_global_1.0_ecgate...'
ecaccess-job-submit -queue ecgb flex_ecmwf_91_global_1.0_ecgate

echo 'submitting example  on demand script flex_ecmwf_91_eta_ecgate...'
ecaccess-job-submit -queue ecgb flex_ecmwf_91_eta_ecgate

echo 'submitting example  operational script ecmwf_idc_ops_ecgate-anfc3hr-gauss...'
ecaccess-job-submit -queue ecgb ecmwf_idc_ops_ecgate-anfc3hr-gauss

echo 'submitting example  operational script ecmwf_idc_ops_ecgate-anfc3hr-799...'
ecaccess-job-submit -queue ecgb ecmwf_idc_ops_ecgate-anfcan3hr-799

else
echo "Expanding templates in local directory"

sed "s,spatlh00/,${ECPATH}," ecmwf_idc_ops_ecgate-anfc3hr-gauss.tmp | sed "s,lh0,${ECUID}," > ecmwf_idc_ops_ecgate-anfc3hr-gauss
sed -i '/queue/ r header' ecmwf_idc_ops_ecgate-anfc3hr-gauss
sed "s,spatlh00/,${ECPATH}," ecmwf_idc_ops_ecgate-anfcan3hr-799.tmp | sed "s,lh0,${ECUID}," > ecmwf_idc_ops_ecgate-anfcan3hr-799
sed -i '/queue/ r header' ecmwf_idc_ops_ecgate-anfcan3hr-799
sed "s,spatlh00/,${ECPATH}," ecmwf_idc_ops_ecgate-anfc3hr-gauss-399.tmp | sed "s,lh0,${ECUID}," >ecmwf_idc_ops_ecgate-anfc3hr-gauss-399
sed -i '/queue/ r header' ecmwf_idc_ops_ecgate-anfc3hr-gauss-399
sed "s,lh0,${ECUID},"  CONTROL_OPS_V2.tmp | sed "s,srvx9.img.univie.ac.at,${GATEWAY}," | sed "s,leo@genericSftp,${DESTINATION}," >CONTROL_OPS_V2
sed  "s,lh0,${ECUID},"  CONTROL_OPS_V4.tmp | sed "s,srvx9.img.univie.ac.at,${GATEWAY}," | sed "s,leo@genericSftp,${DESTINATION}," >CONTROL_OPS_V4

# automatically set gateway and destination for on demand scripts
sed "s,srvx9.img.univie.ac.at,${GATEWAY}," flex_ecmwf_91_finegrid_hpce.tmp | sed "s,leo@genericSftp,${DESTINATION}," > flex_ecmwf_91_finegrid_hpce
sed "s,srvx9.img.univie.ac.at,${GATEWAY}," flex_ecmwf_91_finegrid_ecgate.tmp | sed "s,leo@genericSftp,${DESTINATION}," > flex_ecmwf_91_finegrid_ecgate
sed -i '/queue/ r header' flex_ecmwf_91_finegrid_ecgate
sed "s,srvx9.img.univie.ac.at,${GATEWAY}," flex_ecmwf_91_eta_ecgate.tmp | sed "s,leo@genericSftp,${DESTINATION}," > flex_ecmwf_91_eta_ecgate
sed -i '/queue/ r header' flex_ecmwf_91_eta_ecgate
sed "s,srvx9.img.univie.ac.at,${GATEWAY}," flex_ecmwf_91_global_1.0_ecgate.tmp | sed "s,leo@genericSftp,${DESTINATION}," > flex_ecmwf_91_global_1.0_ecgate
sed -i '/queue/ r header' flex_ecmwf_91_global_1.0_ecgate

mkdir $SCRATCH/ms_sms_output_V4


echo 'submitting example  on demand script flex_ecmwf_91_global_1.0_ecgate...'
ecaccess-job-submit -queue ecgb flex_ecmwf_91_global_1.0_ecgate

echo 'submitting example  on demand script flex_ecmwf_91_eta_ecgate...'
ecaccess-job-submit -queue ecgb flex_ecmwf_91_eta_ecgate

echo 'submitting example  operational script ecmwf_idc_ops_ecgate-anfc3hr-gauss...'
ecaccess-job-submit -queue ecgb ecmwf_idc_ops_ecgate-anfc3hr-gauss

echo 'submitting example  operational script ecmwf_idc_ops_ecgate-anfc3hr-799...'
ecaccess-job-submit -queue ecgb ecmwf_idc_ops_ecgate-anfcan3hr-799

fi
