      subroutine rate(ijx,nx,ny,jpoly,npar,
     1     lsp,lsp9,lsp6)

********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-12-11                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program converts a set of accumulated fluxes   *
* ECMWF model FC fields into equivalent RATES at time ijx          *
* LSP,CP    [mm/hr]                                                *
* SSR,SSHF  [W/(m*m)]                                              *
* EWSS,NSSS [N/(m*m)]                                              *
* This version for fields after 12-09-2000 in which only FC fields *
* with steps of 3, 6, 9 and 12 hours are available due to ECMWFs   *
* 4D-var assimilation system    (ie. no FG fields)                 *
*                                                                  *
********************************************************************

      integer ijx,ijp4,ijpm,i,j,iact,npar
      real step,steprain,steps(npar)

      real lsp(nx,ny,jpoly,npar),lsp6(nx,ny,npar),lsp9(nx,ny,npar)

      NAMELIST /NAMFX2/
     1            NX, NY, 
     2            MAXTIME,JPOLY,
     3            JHRF,
     4            RLO0, RLA0, DX, DY

      call posnam (4,'NAMFX2')
      read (4,NAMFX2)

      if (jhrf.eq.3) then
         step=3600.*3.
         steprain=3./1000.
      else
         step=3600.*6.
         steprain=6./1000.
      end if

      steps(1:2)=steprain
      steps(3:6)=step

      ijp4=mod(ijx,4)
      ijpm=mod(ijx,2)
      iact=ijx
      if (iact.gt.4) iact=4

      print*, jhrf, ijp4, ijpm, iact

      do n=1,npar
      if (jhrf.eq.3) then
* For 3-hourly data
      if (ijp4.eq.1) then
* Fluxes with STEP=03, no change needed
        lsp(:,:,iact,n)=lsp(:,:,iact,n)/steps(n)
      else if (ijp4.eq.2) then
* Fluxes with STEP=06
* First of all, store the normalised STEP=06 values
        lsp6(:,:,n)=lsp(:,:,iact,n)/steps(n)
* Then subtract previous 3-hourly values 
        lsp(:,:,iact,n)=lsp(:,:,iact,n)/steps(n)-lsp(:,:,iact-1,n)
      else if (ijp4.eq.3) then
* Fluxes with STEP=09
* First of all, store the normalised STEP=09 values
        lsp9(:,:,n)=lsp(:,:,iact,n)/steps(n)
* Then subtract previous STEP=06 values 
        lsp(:,:,iact,n)=lsp(:,:,iact,n)/steps(n)-lsp6(:,:,n)
      else
* Fluxes with STEP=12
* Subtract previous STEP=09 values 
        lsp(:,:,iact,n)=lsp(:,:,iact,n)/steps(n)-lsp9(:,:,n)
      end if
      else
* For 6-hourly data
      if (ijpm.eq.1) then
* Fluxes with STEP=03, no change needed
        lsp(:,:,iact,n)=lsp(:,:,iact,n)/steps(n)
      else
* Fluxes with STEP=06
* Subtract previous 3-hourly values 
        lsp(:,:,iact,n)=lsp(:,:,iact,n)/steps(n)-lsp(:,:,iact-1,n)
      end if
      end if
      enddo
      return
      end subroutine rate
