      module flxacc

      contains


      subroutine readflux
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-11-12                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program reads accumulated values of            *
*                                                                  *
*     QUANTITY                     MARS ABBR    CODE               *
*                                                                  *
*     large scale precipitation       LSP       142                *
*     convective precipitation        CP        143                *
*     surface sensible heat flux      SSHF      146                *
*     surface solar radiation         SSR       176                *
*     u stress                        EWSS      180                *
*     v stress                        NSSS      181                * 
*                                                                  *
*                                                                  *
********************************************************************

       USE GRIB_API

      integer kunit(4),ierr,lenght,iword,i,j,itime
      integer idnum(12),itges,ijp
      integer ij,im,id,ih,idel,ind,ic,jmd,hms,jout,ipar,l
      integer iday,ihour,istep

      character*10 fname(4)
      character*1 mode

      double precision jul,juldate


      real, allocatable, dimension(:,:,:,:) :: lsp
      real, allocatable, dimension(:,:,:) :: lsp6
      real, allocatable, dimension(:,:,:) :: lsp9
      real, allocatable, dimension(:,:,:) :: lspi
      real, allocatable, dimension(:) :: values
      integer,allocatable, dimension(:) :: idate
      integer,allocatable, dimension(:,:) :: igrib
      integer :: params(6),npar

      NAMELIST /NAMFX2/
     1            NX, NY, 
     2            MAXTIME,JPOLY,
     3            JHRF,
     4            RLO0, RLA0, DX, DY

C
C Change directory info below, character length must
C be consistent with that above for fname1 etc.
C

      fname=(/'surf_03_ub','surf_06_ub','surf_09_ub','surf_12_ub'/)
      idnum=(/31,0,31,30,31,30,31,31,30,31,30,31/)
      params=(/142,143,146,180,181,176/) !lsp,cp,sshf,ewss,nsss,ssr

      call posnam (4,'NAMFX2')
      read (4,NAMFX2)

      idimens=nx*ny
      npar=6


      allocate(lsp(nx,ny,jpoly,npar))

      allocate(lsp6(nx,ny,npar),lsp9(nx,ny,npar))

      allocate(lspi(nx,ny,npar))

      allocate(idate(maxtime))


      mode='D'
        l=0
        do i=jhrf/6+1,4,jhrf/3
          call grib_open_file(kunit(i),fname(i),'r')
          call grib_count_in_file(kunit(i),n)
        enddo
        allocate(igrib(npar,1))

        do ijp=1,3+1-jhrf/3
* if jhrf=6, then first 3 times are read from kunit2,4,2
* if jhrf=3, then cycles kunit1,2,3,4...
          do j=1,npar
            i=1
            if (jhrf.eq.3) then             
              call grib_new_from_file(kunit(ijp),igrib(j,i), iret)
            else
              call grib_new_from_file(kunit(ijp)*2,igrib(j,i), iret)
            endif

            call grib_get(igrib(j,i),"paramId",ipar)
            if(ipar .ne. params(mod((j-1), npar) +1)) then
              STOP 'unexpected parameter'
            endif
            call grib_get(igrib(j,i),"date",iday)
            call grib_get(igrib(j,i),"time",ihour)
            call grib_get(igrib(j,i),"stepRange",istep)

            jul=juldate(iday,ihour*100)
            jul=jul+dble(istep)/dble(24.0)
            call caldate(jul,jmd,hms)
            idate(ijp)=jmd*100+hms/10000

            call grib_get_size(igrib(j,i),'values',numberOfValues)
*          write(*,*) 'numberOfValues=',numberOfValues
 
            allocate(values(numberOfValues), stat=iret)
      !     get data values
          call grib_get(igrib(j,i),'values',values)
          do k=1,ny
            do l=1,nx
            lsp(l,k,ijp,mod(j-1,npar)+1)=values(nx*(ny-k)+l)
            enddo
          enddo
          write(*,*) ijp,mod(j-1,npar)+1,ipar,iday,ihour,istep,
     *lsp(1,1,ijp,mod(j-1,npar)+1)
        enddo

* Convert fluxes to consistent 3- or 6- hourly RATEs
          call rate(ijp,nx,ny,jpoly,npar,lsp,lsp9,lsp6)
          write(*,*) lsp(1,1,ijp,:)
        enddo

      itime=3
      do while(itime <maxtime)

        itime=itime+1
        do j=1,npar
            i=1
            if (jhrf.eq.3) then             
              call grib_new_from_file(kunit(mod(itime-1,4)+1),igrib(j,i), iret)
            else
              call grib_new_from_file(kunit(mod(itime-1,4)+1)*2,igrib(j,i), iret)
            endif
            if(iret == -1) exit

            call grib_get(igrib(j,i),"paramId",ipar)
            if(ipar .ne. params(mod((j-1), npar) +1)) then
              STOP 'unexpected parameter'
            endif
            call grib_get(igrib(j,i),"date",iday)
            call grib_get(igrib(j,i),"time",ihour)
            call grib_get(igrib(j,i),"stepRange",istep)

            jul=juldate(iday,ihour*100)
            jul=jul+dble(istep)/dble(24.0)
            call caldate(jul,jmd,hms)
            idate(itime)=jmd*100+hms/10000


            call grib_get_size(igrib(j,i),'values',numberOfValues)
*          write(*,*) 'numberOfValues=',numberOfValues
 
            allocate(values(numberOfValues), stat=iret)
      !     get data values
          call grib_get(igrib(j,i),'values',values)
          do k=1,ny
            do l=1,nx
            lsp(l,k,4,mod(j-1,npar)+1)=values(nx*(ny-k)+l)
            enddo
          enddo
!          lsp(:,:,4,mod(j-1,npar)+1)=reshape(values,(/nx,ny/))
          write(*,*) ijp,mod(j-1,npar)+1,ipar,iday,ihour,istep
        enddo

        if(iret == -1) exit
* Convert fluxes to consistent 3- or 6- hourly RATEs
          call rate(itime,nx,ny,jpoly,npar,lsp,lsp9,lsp6)
          write(*,*) lsp(1,1,4,:)

*
* Interpolate deaccumulated fluxes with a cubic-polynomial solution
* or with a modified linear solution for rainfall
      do i=1,2
        call darain(lsp(:,:,:,i),lspi(:,:,i),nx,ny,jpoly)
      enddo
      do i=3,npar
        call dapoly(lsp(:,:,:,i),lspi(:,:,i),nx,ny,jpoly)
      enddo
          write(*,*) 'Int:',lspi(1,1,:)

*
* Write out interpolated / deacc. fields at this timestep
      call writeout(itime,igrib,
     2     lspi,params,idate,
     3     nx,ny,maxtime,idimens,npar)

C *** Shift flux fields back one unit in time
      do i=1,npar
        call daback(lsp(:,:,:,i),nx,ny,jpoly)
      enddo

      enddo ! while

      return
      end subroutine readflux


      subroutine writeout(itime,igrib,
     2     lspi,params,idate,
     3     nx,ny,maxtime,idimens,npar)
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-11-12                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This subroutine writes out the deaccumulated and    *
*              interpolated ECMWF Flux quantities to a grib file   *
*              for every time/date.                                *
*              Note: Negative values are set to zero for all fields*
*              except SSHF                                         *
*                                                                  *
*              filename:  fluxjjjjmmddhh                           *
*                                                                  *
********************************************************************

      use GRIB_API 

      integer i,j,jout,npar
      integer idatehelp,itime
      character date*10,filename*48,mode*1,syscom*80
      integer lunit,ierr      
      character*30 fname
 
      real lspi(nx,ny,npar),values(nx*ny)
      integer idate(maxtime),params(npar)
      integer igrib(npar,1)

      mode='C'
*
      write(date,'(i10.10)') idate(itime-2)
C
C Change the following directory info
C Character length must be consistent with declaration
C for "filename" above
C
      filename='flux'//date
!      call PBOPEN(lunit,filename,'w',ierr)
!      ISEC1(21)=idate(itime-2)/100000000+1
!      idatehelp=mod(idate(itime-2),100000000)
!      ISEC1(10)=idatehelp/1000000
!      ISEC1(11)=idatehelp/10000-ISEC1(10)*100
!      ISEC1(12)=idatehelp/100-ISEC1(10)*10000-ISEC1(11)*100
!      ISEC1(13)=idatehelp-ISEC1(10)*1000000-ISEC1(11)*10000-
!     &                       ISEC1(12)*100

!      print*, idate(itime-2)
!      print*, isec1(10), isec1(11), isec2(12), isec1(13)

*
* WRITE OUT LSP
*
!         ISEC1(38)=1
!         ISEC1(24)=0
!         ISEC1(37)=1
!         ISEC4(2)=16  ! 16bit encoding

      call grib_open_file(LUNIT,filename,'w')
         do n=1,npar

           call grib_set(igrib(n,1),"date",idate(itime-2)/100)
           call grib_set(igrib(n,1),"time",mod(idate(itime-2),100)*100)
           call grib_set(igrib(n,1),"step",0)

           call grib_set(igrib(n,1),"paramId",params(n))
!           zsec4(1:nx*ny)=RESHAPE(lspi(:,:,k),(/nx*ny/))
          do k=1,ny
            do l=1,nx
              values(nx*(ny-k)+l)=lspi(l,k,n)
            enddo
          enddo
          call grib_set(igrib(n,1),"values",values)

           call grib_write(igrib(n,1),Lunit)
         enddo
      call grib_close_file(LUNIT)


      return
      end  subroutine writeout

      subroutine dapoly(xfeld,sfeld,nx,ny,jpoly)
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-03-29                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program interpolates deaccumulated fluxes of   *
* an ECMWF model FG field using a cubic polynomial solution which  *
* conserves the integrals of the fluxes within each timespan       *
* The interpolated values are output at the central point of the 4 *
* accumulation timespans                                           *
*                                                                  *
********************************************************************

      integer i,j
      real xfeld(nx,ny,jpoly),sfeld(nx,ny)
      real pya,pyb,pyc,pyd,xa,xb,xc,xd

      do 400 i=1,nx
      do 400 j=1,ny
         xa=xfeld(i,j,1)
         xb=xfeld(i,j,2)
         xc=xfeld(i,j,3)
         xd=xfeld(i,j,4)
         pya=(xd-xa+3.*(xb-xc))/6.
         pyb=(xc+xa)/2.-xb-9.*pya/2.
         pyc=xb-xa-7.*pya/2.-2.*pyb
         pyd=xa-pya/4.-pyb/3.-pyc/2.
         sfeld(i,j)=8.*pya+4.*pyb+2.*pyc+pyd
400   continue

      return
      end subroutine dapoly

      subroutine daback(xfeld,nx,ny,jpoly)
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-08-11                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************

      integer i,j
      real xfeld(nx,ny,jpoly)

      do 400 i=1,nx
      do 400 j=1,ny
         xfeld(i,j,1)=xfeld(i,j,2)
         xfeld(i,j,2)=xfeld(i,j,3)
         xfeld(i,j,3)=xfeld(i,j,4)
400   continue

      return
      end  subroutine daback

      subroutine darain(xfeld,sfeld,nx,ny,jpoly)
********************************************************************
*                                                                  *
*                     Author: P. JAMES                             *
*                     2000-04-10                                   *
*                     adapted: A. BECK                             *
*                     2003-06-01                                   *
*                                                                  *
********************************************************************
*                                                                  *
* DESCRIPTION: This program interpolates deaccumulated fluxes of   *
* an ECMWF FG rainfall field using a modified linear solution.     *
* The interpolated values are output at the central point of the 4 *
* accumulation timespans                                           *
*                                                                  *
********************************************************************

      integer i,j
      real xfeld(nx,ny,jpoly),sfeld(nx,ny)
      real xa,xb,xc,xd,xac,xbd

      do 400 i=1,nx
      do 400 j=1,ny
         xa=xfeld(i,j,1)
         xb=xfeld(i,j,2)
         xc=xfeld(i,j,3)
         xd=xfeld(i,j,4)
         if (xa.lt.0.) xa=0.
         if (xb.lt.0.) xb=0.
         if (xc.lt.0.) xc=0.
         if (xd.lt.0.) xd=0.
         xac=0.5*xb
         if ((xa+xc).gt.0.) xac=xb*xc/(xa+xc)
         xbd=0.5*xc
         if ((xb+xd).gt.0.) xbd=xb*xc/(xb+xd)
         sfeld(i,j)=xac+xbd
400   continue

      return
      end subroutine darain


      end module flxacc
